@@ -336,11 +336,6 @@ sub check_device {
     my $response = $ua->get($url);
     my $res_time = Time::HiRes::time();
 
-    $hostip ||= 'unknown';
-    $get_port ||= 'unknown';
-    $devid ||= 'unknown';
-    $timeout ||= 'unknown';
-    $url ||= 'unknown';
     unless ($response->is_success) {
         my $failed_after = $res_time - $start_time;
         if ($failed_after < 0.5) {
@@ -457,8 +452,12 @@ sub check_bogus_md5 {
     # most servers /will/ succeed here :<
     my $resp = $self->ua->request($req);
     my $rej = $resp->is_success ? 0 : 1;
-    debug("dev$devid: reject_bad_md5 = $rej");
-    $self->state_event('device', $devid, { reject_bad_md5 => $rej });
+    my $prev = $dev->reject_bad_md5;
+
+    if (!defined($prev) || $prev != $rej) {
+        debug("dev$devid: reject_bad_md5 = $rej");
+        $self->state_event('device', $devid, { reject_bad_md5 => $rej });
+    }
 }
 
 1;
